@@ -25,8 +25,7 @@
  * Derived from Linux version by Richard Tobin.
  * Also partly derived from BSD version.
  *
- * This library is covered by the LGPL.
- *
+ * This library is covered by the LGPL, read LICENSE for details.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -78,24 +77,24 @@ int usb_os_open(usb_dev_handle *dev)
   u_int32_t location;
 
   if (!dev)
-    return -1;
+    USB_ERROR(-ENXIO);
 
   device = malloc(sizeof(struct darwin_dev_handle));
   if (!device)
     USB_ERROR(-ENOMEM);
 
   if (!masterPort)
-    return NULL;
+    USB_ERROR(-EINVAL);
 
   /* set up the matching dictionary for class IOUSBDevice and it's subclasses */
-  if((matchingDict = IOServiceMatching(kIOUSBDeviceClassName)) == NULL){
+  if ((matchingDict = IOServiceMatching(kIOUSBDeviceClassName)) == NULL) {
     mach_port_deallocate(mach_task_self(), masterPort);
-    return NULL;
+    USB_ERROR(-ENOMEM);
   }
 
   gNotifyPort = IONotificationPortCreate(masterPort);
   runLoopSource = IONotificationPortGetRunLoopSource(gNotifyPort);
- 
+
   CFRunLoopAddSource(CFRunLoopGetCurrent(), runLoopSource,
                      kCFRunLoopDefaultMode);
 
@@ -103,7 +102,7 @@ int usb_os_open(usb_dev_handle *dev)
                                             matchingDict, NULL, NULL,
                                             &deviceIterator);
 
-  while (usbDevice = IOIteratorNext(deviceIterator)){
+  while (usbDevice = IOIteratorNext(deviceIterator)) {
     /* Create an intermediate plug-in */
     result = IOCreatePlugInInterfaceForService(usbDevice, kIOUSBDeviceUserClientTypeID,
                                                kIOCFPlugInInterfaceID, &plugInInterface,
@@ -117,7 +116,7 @@ int usb_os_open(usb_dev_handle *dev)
                                        (LPVOID)&darwin_device);
 
     (*plugInInterface)->Release(plugInInterface);
-    
+
     if (!darwin_device)
       continue;
 
@@ -142,11 +141,11 @@ int usb_os_open(usb_dev_handle *dev)
   result = (*(device->device))->USBDeviceOpen(device->device);
 
   if (result)
-    USB_ERROR_STR(result, "could not open device.\n");
-  else
-    dev->impl_info = device;
+    USB_ERROR_STR(result, "could not open device");
 
-  return result;
+  dev->impl_info = device;
+
+  return 0;
 }
 
 int usb_os_close(usb_dev_handle *dev)
@@ -155,10 +154,10 @@ int usb_os_close(usb_dev_handle *dev)
   io_return_t result;
 
   if (!dev)
-    return -1;
+    USB_ERROR(-ENXIO);
 
   if ((device = dev->impl_info) == NULL)
-    return -2;
+    USB_ERROR(-ENOENT);
 
   usb_release_interface(dev, dev->interface);
 
@@ -171,14 +170,11 @@ int usb_os_close(usb_dev_handle *dev)
 
   if (result)
     USB_ERROR_STR(result, "error closing device");
-  
+
   /* device may not need to be released, but if it has too... */
   (*(device->device))->Release(device->device);
 
-  if (result)
-    fprintf(stderr, "error releasing device");
-
-  return result;
+  return 0;
 }
 
 int usb_set_configuration(usb_dev_handle *dev, int configuration)
@@ -187,19 +183,19 @@ int usb_set_configuration(usb_dev_handle *dev, int configuration)
   io_return_t result;
 
   if (!dev)
-    return -1;
+    USB_ERROR(-ENXIO);
 
   if ((device = dev->impl_info) == NULL)
-    return -2;
+    USB_ERROR(-ENOENT);
 
   result = (*(device->device))->SetConfiguration(device->device, configuration);
 
   if (result)
-    USB_ERROR_STR(result, "could not set configuration\n");
+    USB_ERROR_STR(result, "could not set configuration");
 
   dev->config = configuration;
 
-  return result;
+  return 0;
 }
 
 int usb_claim_interface(usb_dev_handle *dev, int interface)
@@ -207,7 +203,7 @@ int usb_claim_interface(usb_dev_handle *dev, int interface)
   struct darwin_dev_handle *device = dev->impl_info;
 
   io_iterator_t interface_iterator;
-  io_service_t  usbInterface;  
+  io_service_t  usbInterface;
 
   io_return_t result;
   IOUSBFindInterfaceRequest request;
@@ -222,13 +218,13 @@ int usb_claim_interface(usb_dev_handle *dev, int interface)
   request.bAlternateSetting = kIOUSBFindInterfaceDontCare;
 
   if (!device)
-    return -1;
+    USB_ERROR(-ENOENT);
 
   if (!(device->device))
-    return -2;
+    USB_ERROR(-EINVAL);
 
   (*(device->device))->CreateInterfaceIterator(device->device, &request, &interface_iterator);
-  
+
   while (current_interface++ <= interface)
     usbInterface = IOIteratorNext(interface_iterator);
 
@@ -237,18 +233,18 @@ int usb_claim_interface(usb_dev_handle *dev, int interface)
   interface_iterator = NULL; /* not really needed */
 
   if (!usbInterface)
-    return -1;
+    USB_ERROR(-ENOENT);
 
   result = IOCreatePlugInInterfaceForService(usbInterface,
 						   kIOUSBInterfaceUserClientTypeID,
 						   kIOCFPlugInInterfaceID,
 						   &plugInInterface, &score);
-  
+
   //No longer need the usbInterface object after getting the plug-in
   result = IOObjectRelease(usbInterface);
   if (result || !plugInInterface)
-    return -1;
-  
+    USB_ERROR(-ENOENT);
+
   //Now create the device interface for the interface
   result = (*plugInInterface)->QueryInterface(plugInInterface,
                                               CFUUIDGetUUIDBytes(kIOUSBInterfaceInterfaceID),
@@ -256,23 +252,27 @@ int usb_claim_interface(usb_dev_handle *dev, int interface)
   //No longer need the intermediate plug-in
   (*plugInInterface)->Release(plugInInterface);
 
-  if (result || !device->interface)
-    return -1;
-  
+  if (result)
+    USB_ERROR(result);
+
+  if (!device->interface)
+    USB_ERROR(-EACCES);
+
   /* claim the interface */
   result = (*(device->interface))->USBInterfaceOpen(device->interface);
   if (result)
-    return -1;
+    USB_ERROR(result);
 
   result = (*(device->interface))->CreateInterfaceAsyncEventSource(device->interface, &runLoopSource);
   if (result)
-    return -1;
+    USB_ERROR(result);
+
   CFRunLoopAddSource(CFRunLoopGetCurrent(), runLoopSource, kCFRunLoopDefaultMode);
 
   dev->interface = interface;
 
   /* interface is claimed and async IO is set up return 0 */
-  return result;
+  return 0;
 }
 
 int usb_release_interface(usb_dev_handle *dev, int interface)
@@ -281,15 +281,15 @@ int usb_release_interface(usb_dev_handle *dev, int interface)
   io_return_t result;
 
   if (!dev)
-    return -1;
+    USB_ERROR(-ENXIO);
 
   if ((device = dev->impl_info) == NULL)
-    return -2;
+    USB_ERROR(-ENOENT);
 
   /* interface is not open */
   if (!device->interface)
     return 0;
- 
+
   result = (*(device->interface))->USBInterfaceClose(device->interface);
 
   if (result)
@@ -302,7 +302,7 @@ int usb_release_interface(usb_dev_handle *dev, int interface)
 
   device->interface = NULL;
 
-  return result;
+  return 0;
 }
 
 int usb_set_altinterface(usb_dev_handle *dev, int alternate)
@@ -311,29 +311,28 @@ int usb_set_altinterface(usb_dev_handle *dev, int alternate)
   io_return_t result;
 
   if (!dev)
-    return -1;
+    USB_ERROR(-ENXIO);
 
   if ((device = dev->impl_info) == NULL)
-    return -2;
+    USB_ERROR(-ENOENT);
 
   /* interface is not open */
-  if (!device->interface) {
-    USB_ERROR_STR(0, "interface used without being claimed");
-    return -1;
-  }
+  if (!device->interface)
+    USB_ERROR_STR(-EACCES, "interface used without being claimed");
 
   result = (*(device->interface))->SetAlternateInterface(device->interface, alternate);
 
   if (result)
-    USB_ERROR_STR(result, "sould not set alternate interface");
+    USB_ERROR_STR(result, "could not set alternate interface");
 
   dev->altsetting = alternate;
 
-  return result;
+  return 0;
 }
 
 /* simple function that figures out what pipeRef is associated with an endpoint */
-static int ep_to_pipeRef (struct darwin_dev_handle *device, int ep) {
+static int ep_to_pipeRef (struct darwin_dev_handle *device, int ep)
+{
   u_int8_t numep, direction, number;
   u_int8_t dont_care1, dont_care3;
   u_int16_t dont_care2;
@@ -354,7 +353,7 @@ static int ep_to_pipeRef (struct darwin_dev_handle *device, int ep) {
     if ( (direction << 7 | number) == ep ) {
       if (usb_debug)
 	fprintf(stderr, "pipeRef for ep 0x%02x found: 0x%02x\n", ep, i);
-      
+
       return i;
     }
   }
@@ -366,13 +365,15 @@ static int ep_to_pipeRef (struct darwin_dev_handle *device, int ep) {
   return -1;
 }
 
-static void write_completed(void *dummy, io_return_t result, void *arg0) {
+static void write_completed(void *dummy, io_return_t result, void *arg0)
+{
   if (usb_debug > 2)
     fprintf(stderr, "write completed\n");
 
   CFRunLoopStop(CFRunLoopGetCurrent());
 }
 
+/* FIXME: Does this need to write chunks like we read chunks? --jerdfelt */
 /* these will require lots of work */
 int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
                    int timeout)
@@ -380,22 +381,20 @@ int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
   struct darwin_dev_handle *device;
   CFRunLoopSourceRef cfSource;
   io_return_t result = -1;
-  int pipeRef, sent = 0;
+  int pipeRef;
 
   if (!dev)
-    return -1;
+    USB_ERROR(-ENXIO);
 
   if ((device = dev->impl_info) == NULL)
-    return -2;
+    USB_ERROR(-ENOENT);
 
   /* interface is not open */
-  if (!device->interface) {
-    USB_ERROR_STR(0, "device used without claiming an interface");
-    return -1;
-  }
+  if (!device->interface)
+    USB_ERROR_STR(-EACCES, "interface used without being claimed");
 
   if ((pipeRef = ep_to_pipeRef(device, ep)) == -1)
-    return -1;
+    USB_ERROR(-EINVAL);
 
   if (usb_debug > 3)
     fprintf(stderr, "usb_bulk_write: endpoint=0x%02x size=%i TO=%i\n", ep, size, timeout);
@@ -405,7 +404,7 @@ int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
 
   /* there seems to be no way to determine how many bytes are actually written */
   result = (*(device->interface))->WritePipeAsyncTO(device->interface, pipeRef,
-					    bytes + (sent), size - sent, 0, timeout,
+					    bytes, size, 0, timeout,
 					    write_completed, NULL);
 
   /* wait for write to complete */
@@ -415,9 +414,9 @@ int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
     fprintf(stderr, "CFLoopRun returned\n");
 
   if (result)
-    USB_ERROR_STR(result, "error writing to device.");
+    USB_ERROR_STR(result, "error writing to device");
 
-  return result;
+  return 0;
 }
 
 int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
@@ -430,19 +429,17 @@ int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
   io_return_t result = -1;
 
   if (!dev)
-    return -1;
+    USB_ERROR(-ENXIO);
 
   if ((device = dev->impl_info) == NULL)
-    return -2;
+    USB_ERROR(-ENOENT);
 
   /* interface is not open */
-  if (!device->interface) {
-    USB_ERROR_STR(0, "interface used without being claimed");
-    return -1;
-  }
+  if (!device->interface)
+    USB_ERROR_STR(-EACCES, "interface used without being claimed");
 
   if ((pipeRef = ep_to_pipeRef(device, ep)) == -1)
-    return -1;
+    USB_ERROR(-EINVAL);
 
   if (usb_debug > 3)
     fprintf(stderr, "usb_bulk_read: endpoint=0x%02x size=%i TO=%i\n", ep, size, timeout);
@@ -451,7 +448,7 @@ int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
      the bulk endpoint */
   do {
     result = (*(device->interface))->ReadPipeTO(device->interface, pipeRef,
-					     bytes + (retrieved), (UInt32 *)&ret_size, 0, timeout);
+					     bytes + retrieved, (UInt32 *)&ret_size, 0, timeout);
     if (result)
       USB_ERROR_STR(result, "error reading from bulk endpoint %02x", ep);
 
@@ -459,15 +456,15 @@ int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
     ret_size = size - retrieved;
   } while (!result && retrieved < size);
 
-  if ( result || (ret_size < 0) )
-    USB_ERROR_STR(0, "error reading from endpoint %02x", ep);
-  
+  if (result || (ret_size < 0))
+    USB_ERROR_STR(result ? result : ret_size, "error reading from endpoint %02x", ep);
+
   retrieved += ret_size;
-  
+
   return retrieved;
 }
 
-int usb_control_msg (usb_dev_handle *dev, int requesttype, int request,
+int usb_control_msg(usb_dev_handle *dev, int requesttype, int request,
                      int value, int index, char *bytes, int size, int timeout)
 {
   struct darwin_dev_handle *device = dev->impl_info;
@@ -491,135 +488,17 @@ int usb_control_msg (usb_dev_handle *dev, int requesttype, int request,
 
   result = (*(device->device))->DeviceRequestTO(device->device, &urequest);
 
-  if (result) {
-    USB_ERROR_STR(result, "error sending control message.");
-    return -1;
-  }
+  if (result)
+    USB_ERROR_STR(result, "error sending control message");
 
   /* i am pretty sure DeviceRequest modifies this */
   return urequest.wLength;
 }
 
-int usb_find_devices_on_bus(struct usb_bus *bus, struct usb_device **devices)
+int usb_os_find_busses(struct usb_bus **busses)
 {
-  struct usb_device *fdev = NULL;
-  io_iterator_t deviceIterator;
-  io_service_t usbDevice;
+  struct usb_bus *fbus = NULL;
 
-  usb_device_t **device;
-
-  IOCFPlugInInterface **plugInInterface = NULL;
-  IONotificationPortRef gNotifyPort;
-
-  CFMutableDictionaryRef matchingDict;
-
-  io_return_t result;
-  SInt32 score;
-  UInt16 address;
-  UInt32 location;
-  UInt32 bus_loc = atoi(bus->dirname);
-
-  /* for use in retrieving device description */
-  IOUSBDevRequest req;
-
-  char buf[20];
-  int busnum = 0;
-
-  if (bus->devices)
-    return 0;
-
-  /* a master port should have been created by find_busses */
-  if (!masterPort)
-    return NULL;
-
-  /* set up the matching dictionary for class IOUSBDevice and it's subclasses */
-  if((matchingDict = IOServiceMatching(kIOUSBDeviceClassName)) == NULL){
-    mach_port_deallocate(mach_task_self(), masterPort);
-    return NULL;
-  }
-
-  gNotifyPort = IONotificationPortCreate(masterPort);
-  runLoopSource = IONotificationPortGetRunLoopSource(gNotifyPort);
- 
-  CFRunLoopAddSource(CFRunLoopGetCurrent(), runLoopSource,
-                     kCFRunLoopDefaultMode);
-
-  result = IOServiceAddMatchingNotification(gNotifyPort, kIOFirstMatchNotification,
-                                            matchingDict, NULL, NULL,
-                                            &deviceIterator);
-
-  req.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBStandard, kUSBDevice);
-  req.bRequest = kUSBRqGetDescriptor;
-  req.wValue = kUSBDeviceDesc << 8;
-  req.wIndex = 0;
-  req.wLength = sizeof(IOUSBDeviceDescriptor);
-
-  while (usbDevice = IOIteratorNext(deviceIterator)){
-    /* Create an intermediate plug-in */
-    result = IOCreatePlugInInterfaceForService(usbDevice, kIOUSBDeviceUserClientTypeID,
-                                               kIOCFPlugInInterfaceID, &plugInInterface,
-                                               &score);
-
-    result = IOObjectRelease(usbDevice);
-    if (result || !plugInInterface)
-      continue;
-
-    (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID),
-                                       (LPVOID)&device);
-
-    /* done with this */
-    (*plugInInterface)->Release(plugInInterface);
-
-    if (!device)
-      continue;
-
-    result = (*(device))->GetDeviceAddress(device, (USBDeviceAddress *)&address);
-    result = (*(device))->GetLocationID(device, &location);
-
-    if ( (location & 0x0f000000) == ((bus_loc+0x8) << 24) ) {
-      struct usb_device *dev;
-      
-      dev = malloc(sizeof(*dev));
-      if (!dev)
-	USB_ERROR(-ENOMEM);
-      
-      memset((void *)dev, 0, sizeof(*dev));
-      
-      dev->bus = bus;
-
-      sprintf(dev->filename, "%03i", address);
-      req.pData = &(dev->descriptor);
-      result = (*(device))->DeviceRequest(device, &req);
-
-      USB_LE16_TO_CPU(dev->descriptor.bcdUSB);
-      USB_LE16_TO_CPU(dev->descriptor.idVendor);
-      USB_LE16_TO_CPU(dev->descriptor.idProduct);
-      USB_LE16_TO_CPU(dev->descriptor.bcdDevice);
-
-      dev->dev = (USBDeviceAddress *)malloc(4);
-      memcpy(dev->dev, &location, 4);
-
-      LIST_ADD(fdev, dev);
-      
-      if (usb_debug >= 2)
-	fprintf(stderr, "usb_find_devices_on_bus: Found %s on %s\n",
-		dev->filename, bus->dirname);
-
-      /* release the device now */
-      (*(device))->Release(device);
-    }
-      
-  }
-
-  IOObjectRelease(deviceIterator);
-
-  *devices = fdev;
-
-  return 0;
-}
-
-int usb_find_busses(void)
-{
   io_iterator_t deviceIterator;
   io_service_t usbDevice;
 
@@ -638,28 +517,24 @@ int usb_find_busses(void)
   char buf[20];
   int busnum = 0;
 
-  /* function was already called */
-  if (usb_busses)
-    return 0;
-
   /* Create a master port for communication with IOKit (this should
    have been created if the user called usb_init() )*/
   if (!masterPort) {
     result = IOMasterPort(MACH_PORT_NULL, &masterPort);
 
     if (result || !masterPort)
-      return NULL;
+      USB_ERROR(-ENOENT);
   }
 
   /* set up the matching dictionary for class IOUSBDevice and it's subclasses */
-  if((matchingDict = IOServiceMatching(kIOUSBDeviceClassName)) == NULL){
+  if ((matchingDict = IOServiceMatching(kIOUSBDeviceClassName)) == NULL) {
     mach_port_deallocate(mach_task_self(), masterPort);
-    return NULL;
+    USB_ERROR(-ENOMEM);
   }
 
   gNotifyPort = IONotificationPortCreate(masterPort);
   runLoopSource = IONotificationPortGetRunLoopSource(gNotifyPort);
-  
+
   CFRunLoopAddSource(CFRunLoopGetCurrent(), runLoopSource,
                      kCFRunLoopDefaultMode);
 
@@ -667,7 +542,7 @@ int usb_find_busses(void)
                                             matchingDict, NULL, NULL,
                                             &deviceIterator);
 
-  while (usbDevice = IOIteratorNext(deviceIterator)){
+  while (usbDevice = IOIteratorNext(deviceIterator)) {
     struct usb_bus *bus;
 
     /* Create an intermediate plug-in */
@@ -691,105 +566,213 @@ int usb_find_busses(void)
     result = (*(device))->GetDeviceAddress(device, (USBDeviceAddress *)&address);
     result = (*(device))->GetLocationID(device, &location);
 
-    if ( address == 0x1 ) {
+    if (address == 0x1) {
       bus = malloc(sizeof(*bus));
       if (!bus)
 	USB_ERROR(-ENOMEM);
-      
+
       memset((void *)bus, 0, sizeof(*bus));
-      
+
       sprintf(buf, "%03i", ( (location & 0x0f000000) >> 24) - 0x8);
       strncpy(bus->dirname, buf, sizeof(bus->dirname) - 1);
       bus->dirname[sizeof(bus->dirname) - 1] = 0;
-      
-      if (usb_busses) {
-	bus->next = usb_busses;
-	bus->next->prev = bus;
-      }
-      usb_busses = bus;
+
+      LIST_ADD(fbus, bus);
 
       if (usb_debug >= 2)
-	fprintf(stderr, "usb_find_busses: Found %s\n", bus->dirname);
+	fprintf(stderr, "usb_os_find_busses: Found %s\n", bus->dirname);
 
       (*(device))->Release(device);
     }
-      
   }
 
   IOObjectRelease(deviceIterator);
 
+  *busses = fbus;
+
+  return 0;
+}
+
+int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
+{
+  struct usb_device *fdev = NULL;
+
+  io_iterator_t deviceIterator;
+  io_service_t usbDevice;
+
+  usb_device_t **device;
+
+  IOCFPlugInInterface **plugInInterface = NULL;
+  IONotificationPortRef gNotifyPort;
+
+  CFMutableDictionaryRef matchingDict;
+
+  io_return_t result;
+  SInt32 score;
+  UInt16 address;
+  UInt32 location;
+  UInt32 bus_loc = atoi(bus->dirname);
+
+  /* for use in retrieving device description */
+  IOUSBDevRequest req;
+
+  char buf[20];
+  int busnum = 0;
+
+  /* a master port should have been created by usb_os_init */
+  if (!masterPort)
+    USB_ERROR(-ENOENT);
+
+  /* set up the matching dictionary for class IOUSBDevice and it's subclasses */
+  if ((matchingDict = IOServiceMatching(kIOUSBDeviceClassName)) == NULL) {
+    mach_port_deallocate(mach_task_self(), masterPort);
+    USB_ERROR(-ENOMEM);
+  }
+
+  gNotifyPort = IONotificationPortCreate(masterPort);
+  runLoopSource = IONotificationPortGetRunLoopSource(gNotifyPort);
+
+  CFRunLoopAddSource(CFRunLoopGetCurrent(), runLoopSource,
+                     kCFRunLoopDefaultMode);
+
+  result = IOServiceAddMatchingNotification(gNotifyPort, kIOFirstMatchNotification,
+                                            matchingDict, NULL, NULL,
+                                            &deviceIterator);
+
+  req.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBStandard, kUSBDevice);
+  req.bRequest = kUSBRqGetDescriptor;
+  req.wValue = kUSBDeviceDesc << 8;
+  req.wIndex = 0;
+  req.wLength = sizeof(IOUSBDeviceDescriptor);
+
+  while (usbDevice = IOIteratorNext(deviceIterator)) {
+    /* Create an intermediate plug-in */
+    result = IOCreatePlugInInterfaceForService(usbDevice, kIOUSBDeviceUserClientTypeID,
+                                               kIOCFPlugInInterfaceID, &plugInInterface,
+                                               &score);
+
+    result = IOObjectRelease(usbDevice);
+    if (result || !plugInInterface)
+      continue;
+
+    (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID),
+                                       (LPVOID)&device);
+
+    /* done with this */
+    (*plugInInterface)->Release(plugInInterface);
+
+    if (!device)
+      continue;
+
+    result = (*(device))->GetDeviceAddress(device, (USBDeviceAddress *)&address);
+    result = (*(device))->GetLocationID(device, &location);
+
+    if ((location & 0x0f000000) == ((bus_loc + 0x8) << 24)) {
+      struct usb_device *dev;
+
+      dev = malloc(sizeof(*dev));
+      if (!dev)
+	USB_ERROR(-ENOMEM);
+
+      memset((void *)dev, 0, sizeof(*dev));
+
+      dev->bus = bus;
+
+      sprintf(dev->filename, "%03i", address);
+      req.pData = &(dev->descriptor);
+      result = (*(device))->DeviceRequest(device, &req);
+
+      USB_LE16_TO_CPU(dev->descriptor.bcdUSB);
+      USB_LE16_TO_CPU(dev->descriptor.idVendor);
+      USB_LE16_TO_CPU(dev->descriptor.idProduct);
+      USB_LE16_TO_CPU(dev->descriptor.bcdDevice);
+
+      dev->dev = (USBDeviceAddress *)malloc(4);
+      memcpy(dev->dev, &location, 4);
+
+      LIST_ADD(fdev, dev);
+
+      if (usb_debug >= 2)
+	fprintf(stderr, "usb_os_find_devices: Found %s on %s\n",
+		dev->filename, bus->dirname);
+
+      /* release the device now */
+      (*(device))->Release(device);
+    }
+  }
+
+  IOObjectRelease(deviceIterator);
+
+  *devices = fdev;
+
   return 0;
 }
 
 void usb_os_init(void)
 {
-    IOMasterPort(MACH_PORT_NULL, &masterPort);
+  IOMasterPort(MACH_PORT_NULL, &masterPort);
 }
 
-int usb_resetep (usb_dev_handle *dev, unsigned int ep)
+int usb_resetep(usb_dev_handle *dev, unsigned int ep)
 {
   struct darwin_dev_handle *device;
   io_return_t result = -1;
 
   if (!dev)
-    return -1;
+    USB_ERROR(-ENXIO);
 
   if ((device = dev->impl_info) == NULL)
-    return -2;
+    USB_ERROR(-ENOENT);
 
   /* interface is not open */
-  if (!device->interface) {
-    USB_ERROR_STR(0, "interface used without being claimed");
-    return -1;
-  }
+  if (!device->interface)
+    USB_ERROR_STR(-EACCES, "interface used without being claimed");
 
   result = (*(device->interface))->ResetPipe(device->interface, ep);
 
   if (result)
     USB_ERROR_STR(result, "error resetting pipe");
 
-  return result;
+  return 0;
 }
 
-int usb_clear_halt (usb_dev_handle *dev, unsigned int ep)
+int usb_clear_halt(usb_dev_handle *dev, unsigned int ep)
 {
   struct darwin_dev_handle *device;
   io_return_t result = -1;
 
   if (!dev)
-    return -1;
+    USB_ERROR(-ENXIO);
 
   if ((device = dev->impl_info) == NULL)
-    return -2;
+    USB_ERROR(-ENOENT);
 
   /* interface is not open */
-  if (!device->interface) {
-    USB_ERROR_STR(0, "interface used without being claimed");
-    return -1;
-  }
+  if (!device->interface)
+    USB_ERROR_STR(-EACCES, "interface used without being claimed");
 
   result = (*(device->interface))->ClearPipeStall(device->interface, ep);
 
   if (result)
     USB_ERROR_STR(result, "error clearing pipe stall");
 
-  return result;
+  return 0;
 }
 
-int usb_reset (usb_dev_handle *dev)
+int usb_reset(usb_dev_handle *dev)
 {
   struct darwin_dev_handle *device;
 
   if (!dev)
-    return -1;
+    USB_ERROR(-ENXIO);
 
   if ((device = dev->impl_info) == NULL)
-    return -1;
+    USB_ERROR(-ENOENT);
 
   if (device->device)
     (*(device->device))->ResetDevice(device->device);
   else
-    USB_ERROR_STR(-1, "no such device");
+    USB_ERROR_STR(-ENOENT, "no such device");
 
   return 0;
 }
