@@ -1,7 +1,7 @@
 /*
  * Linux USB support
  *
- * Copyright (c) 2000-2001 Johannes Erdfelt <johannes@erdfelt.com>
+ * Copyright (c) 2000-2002 Johannes Erdfelt <johannes@erdfelt.com>
  *
  * This library is covered by the LGPL, read LICENSE for details.
  */
@@ -9,9 +9,9 @@
 #include <stdlib.h>	/* getenv, etc */
 #include <unistd.h>
 #include <string.h>
-#include <stdio.h> 
-#include <fcntl.h> 
-#include <errno.h> 
+#include <stdio.h>
+#include <fcntl.h>
+#include <errno.h>
 #include <sys/time.h>
 #include <dirent.h>
 
@@ -89,7 +89,7 @@ int usb_release_interface(usb_dev_handle *dev, int interface)
 
   ret = ioctl(dev->fd, IOCTL_USB_RELEASEINTF, &interface);
   if (ret < 0)
-    USB_ERROR_STR(-errno, "could not release intf %d: %s\n", interface,
+    USB_ERROR_STR(-errno, "could not release intf %d: %s", interface,
     	strerror(errno));
 
   dev->interface = -1;
@@ -118,6 +118,8 @@ int usb_set_altinterface(usb_dev_handle *dev, int alternate)
   return 0;
 }
 
+/* Linux usbdevfs has a limit of one page size per read/write. 4096 is */
+/* the most portable maximum we can do for now */
 #define MAX_READ_WRITE	4096
 
 int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int length,
@@ -136,7 +138,7 @@ int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int length,
       bulk.len = MAX_READ_WRITE;
     bulk.timeout = timeout;
     bulk.data = (unsigned char *)bytes + sent;
-	
+
     ret = ioctl(dev->fd, IOCTL_USB_BULK, &bulk);
     if (ret < 0)
       USB_ERROR_STR(ret, "error writing to bulk endpoint %d: %s",
@@ -144,7 +146,7 @@ int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int length,
 
     sent += ret;
   } while (ret > 0 && sent < length);
-  
+
   return sent;
 }
 
@@ -153,7 +155,7 @@ int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
 {
   struct usb_bulktransfer bulk;
   int ret, retrieved = 0, requested;
-  
+
   /* Ensure the endpoint address is correct */
   ep |= USB_ENDPOINT_IN;
 
@@ -199,7 +201,54 @@ int usb_control_msg(usb_dev_handle *dev, int requesttype, int request,
   return ret;
 }
 
-int usb_find_devices_on_bus(struct usb_bus *bus, struct usb_device **devices)
+int usb_os_find_busses(struct usb_bus **busses)
+{
+  struct usb_bus *fbus = NULL;
+  DIR *dir;
+  struct dirent *entry;
+
+  dir = opendir(usb_path);
+  if (!dir)
+    USB_ERROR_STR(-errno, "couldn't opendir(%s): %s", usb_path,
+	strerror(errno));
+
+  while ((entry = readdir(dir)) != NULL) {
+    struct usb_bus *bus;
+
+    /* Skip anything starting with a . */
+    if (entry->d_name[0] == '.')
+      continue;
+
+    if (!strchr("0123456789", entry->d_name[strlen(entry->d_name) - 1])) {
+      if (usb_debug >= 2)
+        fprintf(stderr, "usb_os_find_busses: Skipping non bus directory %s\n",
+		entry->d_name);
+      continue;
+    }
+
+    bus = malloc(sizeof(*bus));
+    if (!bus)
+      USB_ERROR(-ENOMEM);
+
+    memset((void *)bus, 0, sizeof(*bus));
+
+    strncpy(bus->dirname, entry->d_name, sizeof(bus->dirname) - 1);
+    bus->dirname[sizeof(bus->dirname) - 1] = 0;
+
+    LIST_ADD(fbus, bus);
+
+    if (usb_debug >= 2)
+       fprintf(stderr, "usb_os_find_busses: Found %s\n", bus->dirname);
+  }
+
+  closedir(dir);
+
+  *busses = fbus;
+
+  return 0;
+}
+
+int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
 {
   struct usb_device *fdev = NULL;
   DIR *dir;
@@ -237,7 +286,7 @@ int usb_find_devices_on_bus(struct usb_bus *bus, struct usb_device **devices)
     fd = open(filename, O_RDONLY);
     if (fd < 0) {
       if (usb_debug >= 2)
-        fprintf(stderr, "usb_find_devices_on_bus: Couldn't open %s\n",
+        fprintf(stderr, "usb_os_find_devices: Couldn't open %s\n",
 		filename);
 
       free(dev);
@@ -247,7 +296,7 @@ int usb_find_devices_on_bus(struct usb_bus *bus, struct usb_device **devices)
     ret = read(fd, (void *)&dev->descriptor, sizeof(dev->descriptor));
     if (ret < 0) {
       if (usb_debug)
-        fprintf(stderr, "usb_find_devices_on_bus: Couldn't read descriptor\n");
+        fprintf(stderr, "usb_os_find_devices: Couldn't read descriptor\n");
 
       free(dev);
 
@@ -257,7 +306,7 @@ int usb_find_devices_on_bus(struct usb_bus *bus, struct usb_device **devices)
     LIST_ADD(fdev, dev);
 
     if (usb_debug >= 2)
-      fprintf(stderr, "usb_find_devices_on_bus: Found %s on %s\n",
+      fprintf(stderr, "usb_os_find_devices: Found %s on %s\n",
 		dev->filename, bus->dirname);
 
     /* Now try to fetch the rest of the descriptors */
@@ -341,66 +390,6 @@ err:
   return 0;
 }
 
-int usb_find_busses(void)
-{
-  DIR *dir;
-  struct dirent *entry;
-
-  dir = opendir(usb_path);
-  if (!dir)
-    USB_ERROR_STR(-errno, "couldn't opendir(%s): %s", usb_path,
-	strerror(errno));
-
-  while ((entry = readdir(dir)) != NULL) {
-    struct usb_bus *bus;
-    int skip = 0;
-
-    /* Skip anything starting with a . */
-    if (entry->d_name[0] == '.')
-      continue;
-
-    if (!strchr("0123456789", entry->d_name[strlen(entry->d_name) - 1])) {
-      if (usb_debug >= 2)
-        fprintf(stderr, "usb_find_busses: Skipping non bus directory %s\n",
-		entry->d_name);
-      continue;
-    }
-
-    /* Don't readd it if we were called multiple times */
-    for (bus = usb_busses; bus; bus = bus->next) {
-      if (!strcmp(bus->dirname, entry->d_name)) {
-        skip = 1;
-        break;
-      }
-    }
-
-    if (skip)
-      continue;
-
-    bus = malloc(sizeof(*bus));
-    if (!bus)
-      USB_ERROR(-ENOMEM);
-
-    memset((void *)bus, 0, sizeof(*bus));
-
-    strncpy(bus->dirname, entry->d_name, sizeof(bus->dirname) - 1);
-    bus->dirname[sizeof(bus->dirname) - 1] = 0;
-
-    if (usb_busses) {
-      bus->next = usb_busses;
-      bus->next->prev = bus;
-    }
-    usb_busses = bus;
-
-    if (usb_debug >= 2)
-       fprintf(stderr, "usb_find_busses: Found %s\n", bus->dirname);
-  }
-
-  closedir(dir);
-
-  return 0;
-}
-
 static int check_usb_vfs(const unsigned char *dirname)
 {
   DIR *dir;
@@ -456,37 +445,37 @@ void usb_os_init(void)
   }
 }
 
-int usb_resetep (usb_dev_handle *dev, unsigned int ep)
+int usb_resetep(usb_dev_handle *dev, unsigned int ep)
 {
   int ret;
 
   ret = ioctl(dev->fd, IOCTL_USB_RESETEP, &ep);
   if (ret)
-    USB_ERROR_STR(ret, "could not reset ep %d : %s", ep,
+    USB_ERROR_STR(ret, "could not reset ep %d: %s", ep,
     	strerror(errno));
 
   return 0;
 }
 
-int usb_clear_halt (usb_dev_handle *dev, unsigned int ep)
+int usb_clear_halt(usb_dev_handle *dev, unsigned int ep)
 {
   int ret;
 
   ret = ioctl(dev->fd, IOCTL_USB_CLEAR_HALT, &ep);
   if (ret)
-    USB_ERROR_STR(ret, "could not clear/halt ep %d : %s", ep,
+    USB_ERROR_STR(ret, "could not clear/halt ep %d: %s", ep,
     	strerror(errno));
 
   return 0;
 }
 
-int usb_reset (usb_dev_handle *dev)
+int usb_reset(usb_dev_handle *dev)
 {
   int ret;
 
   ret = ioctl(dev->fd, IOCTL_USB_RESET, NULL);
   if (ret)
-     USB_ERROR_STR(ret, "could not reset : %s", strerror(errno));
+     USB_ERROR_STR(ret, "could not reset: %s", strerror(errno));
 
   return 0;
 }
